package market

import (
	"context"
	"encoding/json"
	"fmt"
	"trasfer_middleware/cmd/rpc/etc"
	"trasfer_middleware/cmd/rpc/internal/logic/po"
	"trasfer_middleware/cmd/rpc/internal/logic/po/market/types"
	"trasfer_middleware/cmd/rpc/internal/logic/vo"
	"trasfer_middleware/cmd/rpc/pb/transfer"
	"trasfer_middleware/cmd/rpc/pkg/mq"
	"trasfer_middleware/until/common"
	"trasfer_middleware/until/request"
	"trasfer_middleware/until/sysLog"
)

type Market struct {
	Conf *types.MarketConf
}

type MarketRequest struct {
	ctx context.Context
	*Market
	*po.RequestStruct
}

func NewMarket(conf types.MarketConf) *Market {
	return &Market{
		Conf: &conf,
	}
}

func (r *Market) SetData(c context.Context, data map[string]interface{}, config *etc.RockerMqConfig) *MarketRequest {
	p := "-----BEGIN RSA PRIVATE KEY-----\n" + data["sign"].(string) + "\n-----END RSA PRIVATE KEY-----"
	sign, err := common.MarketMakeRsaSign(p, data)
	if err != nil {
		panic(err)
	}
	data["sign"] = sign

	requestBody := make(map[string]string, len(data))
	for key, value := range data {
		requestBody[key] = fmt.Sprintf("%v", value)
	}
	return &MarketRequest{
		ctx:    c,
		Market: r,
		RequestStruct: &po.RequestStruct{
			Config:      *config,
			RequestBody: requestBody,
		},
	}
}

func (r *MarketRequest) request(url string) (*request.Response, error) {
	reqUrl := fmt.Sprintf("%s%s", r.Conf.Host, url)
	req := request.Request{
		Method: "POST",
		Url:    reqUrl,
		Data:   r.RequestBody,
	}
	resp, _ := req.Send()
	//异步存入请求记录

	sendMq := mq.AliyunRocketMq{
		AccessKey:     r.Config.AccessKey,
		SecretKey:     r.Config.SecretKey,
		SecurityToken: r.Config.SecurityToken,
		ServerAddress: r.Config.Host,
	}
	err := sendMq.Produce(r.ctx, r.Config.TopicPrefix+r.Config.Topic.Market.Name, po.SetMqSendDataMarket(r.RequestStruct, &resp, url))
	if err != nil {
		sysLog.LogSendMq(r.ctx, err)
	}
	return &resp, nil
}

func (r *MarketRequest) KeySend() (*transfer.MarketKeySendRes, error) {
	var res transfer.MarketKeySendRes
	req, err := r.request(vo.MARKET_KEY_SEND)
	if err != nil {
		return nil, err
	}
	_ = json.Unmarshal([]byte(req.Text), &res)
	return &res, nil
}

func (r *MarketRequest) KeyDiscard() (*transfer.MarketKeyDiscardRes, error) {
	var res transfer.MarketKeyDiscardRes
	req, err := r.request(vo.MARKET_KEY_DISCARD)
	if err != nil {
		return nil, err
	}
	_ = json.Unmarshal([]byte(req.Text), &res)
	return &res, nil
}

func (r *MarketRequest) Query() (*transfer.MarketQueryRes, error) {
	var res transfer.MarketQueryRes
	req, err := r.request(vo.MARKET_KEY_QUERY)
	if err != nil {
		return nil, err
	}
	_ = json.Unmarshal([]byte(req.Text), &res)
	return &res, nil
}
